# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.settings_get_compliance_settings_response_body import SettingsGetComplianceSettingsResponseBody
from ...types.settings_patch_compliance_settings_response_body import SettingsPatchComplianceSettingsResponseBody
from .raw_client import AsyncRawComplianceClient, RawComplianceClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ComplianceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawComplianceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawComplianceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawComplianceClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsGetComplianceSettingsResponseBody:
        """
        Get organization's compliance settings, including carrier name, office address, and DOT number

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsGetComplianceSettingsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.settings.compliance.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        allow_unregulated_vehicles_enabled: typing.Optional[bool] = OMIT,
        canada_hos_enabled: typing.Optional[bool] = OMIT,
        carrier_name: typing.Optional[str] = OMIT,
        dot_number: typing.Optional[int] = OMIT,
        driver_auto_duty_enabled: typing.Optional[bool] = OMIT,
        edit_certified_logs_enabled: typing.Optional[bool] = OMIT,
        force_manual_location_for_duty_status_changes_enabled: typing.Optional[bool] = OMIT,
        force_review_unassigned_hos_enabled: typing.Optional[bool] = OMIT,
        main_office_formatted_address: typing.Optional[str] = OMIT,
        persistent_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SettingsPatchComplianceSettingsResponseBody:
        """
        Update organization's compliance settings, including carrier name, office address, and DOT number

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        allow_unregulated_vehicles_enabled : typing.Optional[bool]
            [deprecated] Allow Unregulated Vehicles. This setting is deprecated as all organizations can now mark vehicles as unregulated.

        canada_hos_enabled : typing.Optional[bool]
            Enable Canada HOS

        carrier_name : typing.Optional[str]
            Carrier Name / Principal Place of Business Name

        dot_number : typing.Optional[int]
            Carrier US DOT Number

        driver_auto_duty_enabled : typing.Optional[bool]
            Enable Driver Auto-Duty

        edit_certified_logs_enabled : typing.Optional[bool]
            Drivers Can Edit Certified Log

        force_manual_location_for_duty_status_changes_enabled : typing.Optional[bool]
            Force Manual Location For Duty Status Changes

        force_review_unassigned_hos_enabled : typing.Optional[bool]
            Force Review of Unassigned HOS

        main_office_formatted_address : typing.Optional[str]
            Main Office Address / Principal Place of Businesss Address

        persistent_duty_status_enabled : typing.Optional[bool]
            Persistent Duty Status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsPatchComplianceSettingsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.settings.compliance.update()
        """
        _response = self._raw_client.update(
            allow_unregulated_vehicles_enabled=allow_unregulated_vehicles_enabled,
            canada_hos_enabled=canada_hos_enabled,
            carrier_name=carrier_name,
            dot_number=dot_number,
            driver_auto_duty_enabled=driver_auto_duty_enabled,
            edit_certified_logs_enabled=edit_certified_logs_enabled,
            force_manual_location_for_duty_status_changes_enabled=force_manual_location_for_duty_status_changes_enabled,
            force_review_unassigned_hos_enabled=force_review_unassigned_hos_enabled,
            main_office_formatted_address=main_office_formatted_address,
            persistent_duty_status_enabled=persistent_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data


class AsyncComplianceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawComplianceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawComplianceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawComplianceClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsGetComplianceSettingsResponseBody:
        """
        Get organization's compliance settings, including carrier name, office address, and DOT number

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsGetComplianceSettingsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.settings.compliance.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        allow_unregulated_vehicles_enabled: typing.Optional[bool] = OMIT,
        canada_hos_enabled: typing.Optional[bool] = OMIT,
        carrier_name: typing.Optional[str] = OMIT,
        dot_number: typing.Optional[int] = OMIT,
        driver_auto_duty_enabled: typing.Optional[bool] = OMIT,
        edit_certified_logs_enabled: typing.Optional[bool] = OMIT,
        force_manual_location_for_duty_status_changes_enabled: typing.Optional[bool] = OMIT,
        force_review_unassigned_hos_enabled: typing.Optional[bool] = OMIT,
        main_office_formatted_address: typing.Optional[str] = OMIT,
        persistent_duty_status_enabled: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SettingsPatchComplianceSettingsResponseBody:
        """
        Update organization's compliance settings, including carrier name, office address, and DOT number

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write ELD Compliance Settings (US)** under the Compliance category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        allow_unregulated_vehicles_enabled : typing.Optional[bool]
            [deprecated] Allow Unregulated Vehicles. This setting is deprecated as all organizations can now mark vehicles as unregulated.

        canada_hos_enabled : typing.Optional[bool]
            Enable Canada HOS

        carrier_name : typing.Optional[str]
            Carrier Name / Principal Place of Business Name

        dot_number : typing.Optional[int]
            Carrier US DOT Number

        driver_auto_duty_enabled : typing.Optional[bool]
            Enable Driver Auto-Duty

        edit_certified_logs_enabled : typing.Optional[bool]
            Drivers Can Edit Certified Log

        force_manual_location_for_duty_status_changes_enabled : typing.Optional[bool]
            Force Manual Location For Duty Status Changes

        force_review_unassigned_hos_enabled : typing.Optional[bool]
            Force Review of Unassigned HOS

        main_office_formatted_address : typing.Optional[str]
            Main Office Address / Principal Place of Businesss Address

        persistent_duty_status_enabled : typing.Optional[bool]
            Persistent Duty Status

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsPatchComplianceSettingsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.settings.compliance.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            allow_unregulated_vehicles_enabled=allow_unregulated_vehicles_enabled,
            canada_hos_enabled=canada_hos_enabled,
            carrier_name=carrier_name,
            dot_number=dot_number,
            driver_auto_duty_enabled=driver_auto_duty_enabled,
            edit_certified_logs_enabled=edit_certified_logs_enabled,
            force_manual_location_for_duty_status_changes_enabled=force_manual_location_for_duty_status_changes_enabled,
            force_review_unassigned_hos_enabled=force_review_unassigned_hos_enabled,
            main_office_formatted_address=main_office_formatted_address,
            persistent_duty_status_enabled=persistent_duty_status_enabled,
            request_options=request_options,
        )
        return _response.data
