# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.driver_app_settings_gamification_config_tiny_object_request_body import (
    DriverAppSettingsGamificationConfigTinyObjectRequestBody,
)
from ...types.driver_app_settings_trailer_selection_config_tiny_object_request_body import (
    DriverAppSettingsTrailerSelectionConfigTinyObjectRequestBody,
)
from ...types.settings_get_driver_app_settings_response_body import SettingsGetDriverAppSettingsResponseBody
from ...types.settings_patch_driver_app_settings_response_body import SettingsPatchDriverAppSettingsResponseBody
from .raw_client import AsyncRawDriverAppClient, RawDriverAppClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DriverAppClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDriverAppClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDriverAppClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDriverAppClient
        """
        return self._raw_client

    def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsGetDriverAppSettingsResponseBody:
        """
        Get driver app settings.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver App Settings** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsGetDriverAppSettingsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.settings.driver_app.get()
        """
        _response = self._raw_client.get(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        driver_fleet_id: typing.Optional[str] = OMIT,
        gamification: typing.Optional[bool] = OMIT,
        gamification_config: typing.Optional[DriverAppSettingsGamificationConfigTinyObjectRequestBody] = OMIT,
        org_vehicle_search: typing.Optional[bool] = OMIT,
        trailer_selection: typing.Optional[bool] = OMIT,
        trailer_selection_config: typing.Optional[DriverAppSettingsTrailerSelectionConfigTinyObjectRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SettingsPatchDriverAppSettingsResponseBody:
        """
        Update driver app settings.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver App Settings** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_fleet_id : typing.Optional[str]
            Global login user name for the fleet driver app

        gamification : typing.Optional[bool]
            Driver gamification feature. Enabling this will turn on the feature for all drivers using the mobile app. Drivers can be configured into peer groups within the Drivers Page. Unconfigured drivers will be grouped on an organization level.

        gamification_config : typing.Optional[DriverAppSettingsGamificationConfigTinyObjectRequestBody]

        org_vehicle_search : typing.Optional[bool]
            Allow drivers to search for vehicles outside of their selection tag when connected to the internet.

        trailer_selection : typing.Optional[bool]
            Allow drivers to see and select trailers in the Samsara Driver app.

        trailer_selection_config : typing.Optional[DriverAppSettingsTrailerSelectionConfigTinyObjectRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsPatchDriverAppSettingsResponseBody
            OK response.

        Examples
        --------
        from samsara import Samsara

        client = Samsara(
            token="YOUR_TOKEN",
        )
        client.settings.driver_app.update()
        """
        _response = self._raw_client.update(
            driver_fleet_id=driver_fleet_id,
            gamification=gamification,
            gamification_config=gamification_config,
            org_vehicle_search=org_vehicle_search,
            trailer_selection=trailer_selection,
            trailer_selection_config=trailer_selection_config,
            request_options=request_options,
        )
        return _response.data


class AsyncDriverAppClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDriverAppClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDriverAppClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDriverAppClient
        """
        return self._raw_client

    async def get(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SettingsGetDriverAppSettingsResponseBody:
        """
        Get driver app settings.

         <b>Rate limit:</b> 5 requests/sec (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Read Driver App Settings** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsGetDriverAppSettingsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.settings.driver_app.get()


        asyncio.run(main())
        """
        _response = await self._raw_client.get(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        driver_fleet_id: typing.Optional[str] = OMIT,
        gamification: typing.Optional[bool] = OMIT,
        gamification_config: typing.Optional[DriverAppSettingsGamificationConfigTinyObjectRequestBody] = OMIT,
        org_vehicle_search: typing.Optional[bool] = OMIT,
        trailer_selection: typing.Optional[bool] = OMIT,
        trailer_selection_config: typing.Optional[DriverAppSettingsTrailerSelectionConfigTinyObjectRequestBody] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SettingsPatchDriverAppSettingsResponseBody:
        """
        Update driver app settings.

         <b>Rate limit:</b> 100 requests/min (learn more about rate limits <a href="https://developers.samsara.com/docs/rate-limits" target="_blank">here</a>).

        To use this endpoint, select **Write Driver App Settings** under the Drivers category when creating or editing an API token. <a href="https://developers.samsara.com/docs/authentication#scopes-for-api-tokens" target="_blank">Learn More.</a>


         **Submit Feedback**: Likes, dislikes, and API feature requests should be filed as feedback in our <a href="https://forms.gle/zkD4NCH7HjKb7mm69" target="_blank">API feedback form</a>. If you encountered an issue or noticed inaccuracies in the API documentation, please <a href="https://www.samsara.com/help" target="_blank">submit a case</a> to our support team.

        Parameters
        ----------
        driver_fleet_id : typing.Optional[str]
            Global login user name for the fleet driver app

        gamification : typing.Optional[bool]
            Driver gamification feature. Enabling this will turn on the feature for all drivers using the mobile app. Drivers can be configured into peer groups within the Drivers Page. Unconfigured drivers will be grouped on an organization level.

        gamification_config : typing.Optional[DriverAppSettingsGamificationConfigTinyObjectRequestBody]

        org_vehicle_search : typing.Optional[bool]
            Allow drivers to search for vehicles outside of their selection tag when connected to the internet.

        trailer_selection : typing.Optional[bool]
            Allow drivers to see and select trailers in the Samsara Driver app.

        trailer_selection_config : typing.Optional[DriverAppSettingsTrailerSelectionConfigTinyObjectRequestBody]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SettingsPatchDriverAppSettingsResponseBody
            OK response.

        Examples
        --------
        import asyncio

        from samsara import AsyncSamsara

        client = AsyncSamsara(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.settings.driver_app.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            driver_fleet_id=driver_fleet_id,
            gamification=gamification,
            gamification_config=gamification_config,
            org_vehicle_search=org_vehicle_search,
            trailer_selection=trailer_selection,
            trailer_selection_config=trailer_selection_config,
            request_options=request_options,
        )
        return _response.data
